#include "Cryptor.h"

@implementation Cryptor

- (BOOL)encryptFile:(int)plain_fd intoFile:(int)cipher_fd publicKeyString:(char*)userID;
{
    int procID;
    struct stat statbuf;
    char pgpexec[]="/usr/local/bin/pgpe";

    /* Check for NULL userID */
    if(userID==NULL) {
	fprintf(stderr,"Can't encrypt with NULL userID\n");
	return 0;
    }

    /* check if executable exists */
    if(stat(pgpexec,&statbuf)) {
	perror("Error with pgpe:");
	return 0;
    }

    /* Fork a process */
    procID = fork();
    if(procID<0) {
	perror("Cryptor can't fork");
	exit(0);
    }

    if(procID>0) {
	/* Parent waits until child finishes */
	while(wait(NULL) != procID) ;
	close(cipher_fd);
	close(plain_fd);

	return 1;
    }

    /*  Child process here */
    if(plain_fd != 0) {
	/* Dup plain_fd into stdin */
	close(0);
	dup(plain_fd);
    }

    if(cipher_fd != 1) {
	/* Dup cipher_fd into stdout */
	close(1);
	dup(cipher_fd);
    }

    execl(pgpexec, pgpexec, "+batchmode", "+force", "-faq", "-r", userID, NULL);
    return 1;
}

    
- (BOOL)decryptFile:(int)cipher_fd intoFile:(int)plain_fd privateKeyString:(char*)privateKeyPhrase
{
    int procID;
    struct stat statbuf;
    char pgpexec[]="/usr/local/bin/pgpv";

    /* Check for NULL keyPhrase */
    if(privateKeyPhrase==NULL) {
	fprintf(stderr,"Can't decrypt with NULL keyPhrase\n");
	return 0;
    }

    /* check if executable exists */
    if(stat("/usr/local/bin/pgpv",&statbuf)) {
	perror("Error with pgpv:");
	return 0;
    }

    /* Fork a process */
    procID = fork();

    if(procID<0) {
	/* Bad fork */
	perror("Cryptor can't fork");
	exit(1);
    }

    if(procID>0) {
	/* Parent waits until child finishes */
	while(wait(NULL) != procID) ;
	close(cipher_fd);
	close(plain_fd);
	    
	return 1;
    }

    
    /*  Child process here */
    if(cipher_fd != 0) {
	/* Dup cipher_fd into stdin */
	close(0);
	dup(cipher_fd);
    }

    if(plain_fd != 1) {
	/* Dup plain_fd into stdout */
	close(1);
	dup(plain_fd);
    }

    /* Let pgp do the dirty work */
    execl(pgpexec, pgpexec, "+batchmode", "-fq", "-z", privateKeyPhrase, NULL);
    return 1;
}
	

@end
