#import <stdio.h>
#import <stdlib.h>
#import <string.h>
#import <signal.h>
#import <sys/types.h>
#import <sys/stat.h>
#import <sys/param.h>
#import <objc/Object.h>
#import <appkit/appkit.h>
#import <sys/types.h>
#import <sys/stat.h>

#define MAXLINE 132

@interface KeyManager : Object
{
    char **userList;
    int userIDCount;
    char *lastKeyID;
    char *lastKeyFile;
    BOOL pgpInstalled;
}

- init;
- (char *)getKeyForUser:(char *)user;
- (int)userCount;
- (char **)userList;
- writeKey:(char *)key toFilePath:(char *)file;
- keyFromFile:(char *)filename;
- (char *)keyID;
- (char *)mailingAddress;

- adduser:(char *)user keyData:(char *)key keyLength:(int)length;
- removeUser:(char *)userID;

- (char *)getAlternateKeyForUser:(char *)user;
- importKey:sender;
- importSelectionAsKey:(id)pb;
- copyKey:sender;


/*  Not implemented */

- (char *)getPublicKeyFor:(char *)user;
- (char *)publicKeyFromPrivateKey:(char *)privateKey;
- beginKeyGeneration;
- endKeyGeneration;
- addKeyToKeyring:(char *)key;

@end



@interface Menu(Cryptor)
- km_findCellWithTitle:(const char *)title;
- km_addSlashItem:(const char *)title action:(SEL)sel keyEquivalent:(unsigned short)code;
@end

@implementation Menu(Cryptor)

- km_findCellWithTitle:(const char *)title
{
   id cell=nil;
   int x, y, rows, cols;
   const char *c,*end;

   if (title==0) return nil;
   for(end=title;(*end) && (*end!='/');end++);
   
   [matrix getNumRows:&rows numCols:&cols];
   for(y=0;(y<rows) && (cell==nil);y++) for(x=0;(x<cols) && (cell==nil);x++)
   {
      if ((cell=[matrix cellAt:y:x])==nil) continue;
      
      if (((c=[cell title])==0)||(strncmp(title,c,end-title)!=0)||(c[end-title]!='\0'))
         cell=nil;
   }

   if (cell==nil)
      return nil;
   else if ((*end!='\0') && [cell hasSubmenu])
      return [[cell target] km_findCellWithTitle:end+1];
   else if (*end=='\0')
      return cell;
   else
      return nil;
}

- km_addSlashItem:(const char *)title action:(SEL)sel keyEquivalent:(unsigned short)code
{
   id cell=nil;
   int x, y, rows, cols;
   const char *c,*end;

   if (title==0) return nil;
   for(end=title;(*end) && (*end!='/');end++);

   [matrix getNumRows:&rows numCols:&cols];
   for(y=0;(y<rows) && (cell==nil);y++) for(x=0;(x<cols) && (cell==nil);x++)
   {
      if ((cell=[matrix cellAt:y:x])==nil) continue;
      
      if (((c=[cell title])==0)||(strncmp(title,c,end-title)!=0)||(c[end-title]!='\0'))
         cell=nil;
   }

   if ((cell==nil) && (*end=='\0'))
   {
      cell=[self addItem:title action:sel keyEquivalent:code];
      [self display];
      return cell;
   }
   else if (cell==nil)
   {
      char *d=alloca(end-title+1);
      strncpy(d,title,end-title);
      d[end-title]='\0';
      cell=[self addItem:d action:0 keyEquivalent:0];
      [self setSubmenu:[[Menu alloc] initTitle:d] forItem:cell];
      [self display];
      return [[cell target] km_addSlashItem:end+1 action:sel keyEquivalent:code];
   }
   else if ((*end!='\0') && [cell hasSubmenu])
      return [[cell target] km_addSlashItem:end+1 action:sel keyEquivalent:code];
   else if (*end=='\0')
      return cell;
   else
      return nil;
}

@end



/* Error messages */

char noBinsAlertMsg[] =
	"PGP is not installed correctly.  You must insure that  pgpe, pgpv, pgps, and pgpk are all installed in the /usr/local/bin directory";

char noPGPeExecAlertMsg[] ="/usr/local/bin/pgp must have execute permissions.\n\nYou may enable execute permissions with the command \"chmod a+x /usr/local/bin/pgp\" from a Terminal shell";

char noPGPkExecAlertMsg[] ="/usr/local/bin/pgpk must have execute permissions.\n\nYou may enable execute permissions with the command \"chmod a+x /usr/local/bin/pgpk\" from a Terminal shell";

char noHomeDir[] = "Can't find your HOME directory.  Make sure the HOME environment variable has a value";

char noPGPdir[] = "You must have a .pgp directory in your HOME directory.  The .pgp directory should contain your keyrings and the pgp.cfg configuration file";

char pgpNotDir[] = "The .pgp entry in your HOME directory is a file, not a directory.\n\n.pgp should be a directory containing your pgp.cfg configuration file and your keyrings";

char noConfigFile[] = "There is no pgp.cfg file in your .pgp directory.\n\nCopy the sample pgp.cfg file from the Cryptor.bundle distribution into the .pgp directory.  Change the \"MyName\" entry in the pgp.cfg file from \"hcole\" to your user name";

char pgpKeyError[] = "PGP has encountered an error trying to read your keyring.  The error returned from pgp:\n\n%s";

char noKeysOnRing[] = "There are no keys on your keyring.  Generate a new key by executing \"pgpk -g\" from a command line and following the instructions";

char unreadableCFG[] = "Your pgp.cfg file does not have read permission set.  Please set readable permissions with this command:  \"chmod 0100 pgp.cfg\"";


