#import "KeyManager.h"

@implementation KeyManager : Object

- init
{
    FILE *pgp_fd;
    char pgp_in[128];
    struct stat statbuf;
    char *homeDir,pgpDir[512],fileName[512];
    id newCell;

    userList = NULL;
    userIDCount = 0;
    lastKeyID = NULL;
    lastKeyFile = NULL;
    pgpInstalled = NO;
    signal(SIGPIPE,SIG_IGN);

    /* Go through many checks to make sure pgp is installed correctly */

    if((stat("/usr/local/bin/pgpe",&statbuf)==-1) ||
       (stat("/usr/local/bin/pgpv",&statbuf)==-1) ||
       (stat("/usr/local/bin/pgps",&statbuf)==-1) ||
       (stat("/usr/local/bin/pgpk",&statbuf)==-1))  {
	NXRunAlertPanel("Cryptor Warning",noBinsAlertMsg,0,0,0);
	return NULL;
    }

    /* Check for executable permissions on pgp binary */
    stat("/usr/local/bin/pgp",&statbuf);
    if((statbuf.st_mode & 0x49) != 0x49) {
	NXRunAlertPanel("Cryptor Warning",noPGPeExecAlertMsg,0,0,0);
	return NULL;
    }

    /* Check for executable permissions on pgpk binary */
    stat("/usr/local/bin/pgpk",&statbuf);
    if((statbuf.st_mode & 0x49) != 0x49) {
	NXRunAlertPanel("Cryptor Warning",noPGPkExecAlertMsg,0,0,0);
	return NULL;
    }

    /* Get user's home directory and pgp directory */
    homeDir = getenv("HOME");
    if(!homeDir) {
	NXRunAlertPanel("Cryptor Warning",noHomeDir,0,0,0);
	return NULL;
    }
    strcpy(pgpDir,homeDir);

    /* Check for existance of .pgp directory */
    strcpy(fileName,pgpDir);
    strcat(fileName,"/.pgp");

    if(stat(fileName,&statbuf)==-1) {
	NXRunAlertPanel("Cryptor Warning",noPGPdir,0,0,0);
	return NULL;
    }

    /* Check if existing .pgp is really a directory */
    if((statbuf.st_mode & S_IFDIR) != S_IFDIR) {
	NXRunAlertPanel("Cryptor Warning",pgpNotDir,0,0,0);
	return NULL;
    }

    /* Check for pgp.cfg configuration file */
    strcat(fileName,"/pgp.cfg");
    if(stat(fileName,&statbuf)==-1) {
	NXRunAlertPanel("Cryptor Warning",noConfigFile,0,0,0);
	return NULL;
    }

    /* Check if pgp.cfg is readable */
    if((statbuf.st_mode & 0x80) != 0x80) {
	NXRunAlertPanel("Cryptor Warning",unreadableCFG,0,0,0);
	return NULL;
    }

    /* Check for keys on keyring */
    pgp_fd = popen("/usr/local/bin/pgpk -l","r");
    if(!pgp_fd) {
	fprintf(stderr,"Can't open Key Pipe\n");
	return NULL;
    }

    /* Read first line */
    fgets(pgp_in,MAXLINE,pgp_fd);

    /* Read dummy lines until pipe is empty */
    do {
	if(fgets(pgpDir,MAXLINE,pgp_fd)==NULL) break;
    } while(1);
    pclose(pgp_fd);

    /* Now, check first line for error message */
    if(strncmp(pgp_in,"Type",4)) {
	NXRunAlertPanel("Cryptor Warning",pgpKeyError,pgp_in,0,0,0);
	return NULL;
    }

    /* If you get this far, pgp should work correctly */
    pgpInstalled = YES;

    if([self userCount]==0)
      NXRunAlertPanel("Cryptor Warning",noKeysOnRing,0,0,0);


    /* Add import key button to main menu */

    newCell = [[NXApp mainMenu] km_addSlashItem:"Info/Import Key Selection" 
                                   action:@selector(importKey:) keyEquivalent:0];
    [newCell setTarget:self];

    newCell = [[NXApp mainMenu] km_addSlashItem:"Info/Copy Key To Pasteboard" 
                                   action:@selector(copyKey:) keyEquivalent:0];
    [newCell setTarget:self];

    return self;
}


- (char *)getKeyForUser:(char *)user
{
    char *pubKey;
    int iuser,foundUser;
    char pgpexec[2048];

    if(!pgpInstalled) return NULL;
    if(user==NULL) return NULL;

    if(userIDCount==0)
      if([self userCount]==0) return NULL;

    foundUser = 0;
    for(iuser=0; iuser<userIDCount; iuser++) {
	if(strstr(userList[iuser],user)) {
	    foundUser=1;
	    break;
	}
    }
    
    if(!foundUser) return NULL;

    pubKey = malloc(strlen(user));
    strcpy(pubKey,user);

    if(lastKeyID) free(lastKeyID);
    lastKeyID = (char *)malloc(strlen(pubKey)+16);
    strcpy(lastKeyID,pubKey);

    if(lastKeyFile) free(lastKeyFile);
    lastKeyFile = NULL;

    /* Key to temporary file for possible future use */
    sprintf(pgpexec,"/usr/local/bin/pgpk +batchmode +force -xa \"%s\" > /tmp/lastKey.key",user);
    system(pgpexec);

    return pubKey;


}


- (int)userCount
{
    int nusers,iuser;
    char pgp_in[MAXLINE];
    FILE *pgp_fd;
    BOOL lastLineIsUID;

    if(!pgpInstalled) return 0;

    pgp_fd = popen("/usr/local/bin/pgpk -l","r");
    if(!pgp_fd) {
	fprintf(stderr,"Can't open Key Pipe\n");
	return 0;
    }

    /* Read first line, check for error message */
    fgets(pgp_in,MAXLINE,pgp_fd);
    if(strncmp(pgp_in,"Type",4)) {
	putc('\007',stderr);
	fputs(pgp_in,stderr);
	pclose(pgp_fd);
	return 0;
    }

    for(iuser=0; iuser<userIDCount; iuser++)
      free(userList[iuser]);
    free(userList);

    userList = (char **)malloc(sizeof(char *));
    userList[0] = NULL;
    nusers = 0;
    lastLineIsUID = NO;
    do {
	if(fgets(pgp_in,MAXLINE,pgp_fd)==NULL) break;
	if(!strncmp(pgp_in,"uid",3)) {
	    /* Keep only the first UID for each key */
	    if(!lastLineIsUID) {
		int lastchar;
		lastchar = strlen(pgp_in);
		pgp_in[lastchar-1] = '\0';

		userList = (char **)realloc(userList,(nusers+1)*sizeof(char *));
		userList[nusers] = (char *)malloc(strlen(pgp_in)+16);
		strcpy(userList[nusers],pgp_in+5);
		lastLineIsUID = YES;
		
		nusers++;
	    }
	}
	else lastLineIsUID = NO;
    } while(1);

    pclose(pgp_fd);

    userIDCount = nusers;
    return nusers;
}


- (char **)userList
{
    int nusers;

    if(!pgpInstalled) return NULL;
    if(userIDCount) return userList;

    nusers = [self userCount];
    if(nusers) return userList;

    return NULL;
}


- writeKey:(char *)key toFilePath:(char *)filename
{
    FILE *in_key_fd,*out_key_fd;
    char pgpexec[1024],keyLine[256];
    char *keyPath,*userID;
    char *chrptr;

    if(!pgpInstalled) return nil;

    /* Open the output file */
    if(!(out_key_fd=fopen(filename,"w"))) {
	fprintf(stderr,"Cryptor.bundle:  Can't open Key Output File: %s\n",
		filename);
	return nil;
    }

    /* Initialize a userID */
    keyPath = (char *)malloc(strlen(filename)+16);
    strcpy(keyPath,filename);
    
    /* strip off file extension */
    chrptr = strstr(keyPath,".key");
    if(chrptr) *chrptr = '\0';

    /* strip off path */
    chrptr = strrchr(keyPath,'/');
    userID = (char *)malloc(strlen(chrptr)+16);

    if(chrptr) strcpy(userID,chrptr+1);
    else strcpy(userID,keyPath);

    /* Open a pipe to PGP and extract key */
    sprintf(pgpexec,"/usr/local/bin/pgpk -xqa \"%s\"",userID);
    in_key_fd = popen(pgpexec,"r");
    
    /* Get key for userID and write to file */
    do {
	if(fgets(keyLine,248,in_key_fd)==0) break;
	fputs(keyLine,out_key_fd);
    } while(1);

    fclose(out_key_fd);
    pclose(in_key_fd);

    free(userID);
    free(keyPath);
    
    return self;
}


- keyFromFile:(char *)filename
{
    char *keyPath,*chrptr;
    char *userID;
    
    if(!pgpInstalled) return nil;

    if(lastKeyFile) free(lastKeyFile);
    lastKeyFile = (char *)malloc(strlen(filename)+16);
    strcpy(lastKeyFile,filename);
    
    /* Initialize a userID */
    keyPath = (char *)malloc(strlen(filename)+16);
    strcpy(keyPath,filename);

    /* strip off path */
    chrptr = rindex(keyPath,'/');
    userID = (char *)malloc(strlen(keyPath)+16);
    
    if(chrptr)
	strcpy(userID,chrptr+1);
    else
	strcpy(userID,keyPath);

    /* strip off file extension */
    chrptr = strstr(userID,".key");
    if(chrptr) *chrptr = '\0';

    if(lastKeyID) free(lastKeyID);
    lastKeyID = (char *)malloc(strlen(userID)+16);
    strcpy(lastKeyID,userID);

    return self;
}


- (char *)keyID
{
    char *newKeyID;
    
    if(!pgpInstalled) return NULL;
    if(!lastKeyID) return NULL;
    
    newKeyID = (char *)malloc(strlen(lastKeyID)+16);
    strcpy(newKeyID,lastKeyID);
    
    return newKeyID;
}




- (char *)mailingAddress
{
    char *lastAddr;
    
    if(!pgpInstalled) return NULL;
    if(!lastKeyID) return NULL;

    lastAddr = (char *)malloc(strlen(lastKeyID)+16);
    strcpy(lastAddr,lastKeyID);

    return lastAddr;
}


- adduser:(char *)user keyData:(char *)key keyLength:(int)length
{
    char pgpexec[1024];
    
    if(!pgpInstalled) return nil;

    sprintf(pgpexec,"/usr/local/bin/pgpk +batchmode +force -qa \"%s\"",lastKeyFile);
    system(pgpexec);

    return self;
}


- (char *)getAlternateKeyForUser:(char *)user
{
    return NULL;
}


- removeUser:(char *)userID
{
    char pgpexec[1024];

    if(!pgpInstalled) return nil;
    if(!userID) return nil;

    sprintf(pgpexec,"/usr/local/bin/pgpk +batchmode +force -qr \"%s\"",userID);
    system(pgpexec);

    return self;
}


- importKey:sender
{
    id pasteboard = [Pasteboard new];

    [self importSelectionAsKey:pasteboard];
    return self;
}


- importSelectionAsKey:(id)pb
{
    const NXAtom *pbType;              // type of data currently in pasteboard
    char *pbBuffer;                    // data buffers
    int length;                        // length of pb data
    char pgpexec[2048];
    FILE *keyFile;

    if(!pgpInstalled) return nil;

    pbType = [pb types];    // pretend to check the pasteboard types
    
    // read the ASCII data from the pasteboard
    [pb readType:NXAsciiPboardType data:&pbBuffer length:&length];

    // Check for PGP Key block 
    if(!strstr(pbBuffer,"PUBLIC KEY BLOCK")) {
	NXRunAlertPanel("No PGP Key","There is no PGP key in the pasteboard.  Please select the PGP key block and copy it to the pasteboard with the \"Edit : Copy\" menu item.",0,0,0);
	return self;
    }

    // Write key to temporary key file
    if(!(keyFile = fopen("/tmp/tmpkey.key","w"))) {
	NXRunAlertPanel("Can't import PGP Key","Can't create a temporary key file in /tmp",0,0,0);
	return self;
    }

    fwrite(pbBuffer,length,1,keyFile);

    fclose(keyFile);

    unlink("/tmp/tmpkey.key");

    // PGP command to add key
    sprintf(pgpexec,"/usr/local/bin/pgpk +batchmode +force -qa /tmp/tmpkey.key");
    system(pgpexec);

    return self;

}


- copyKey:sender
{
    char *keybuf;
    FILE *key_fd;
    struct stat sbuf;
    int keylength;
    id pb;
    const char *pbTypes[1];

    stat("/tmp/lastKey.key",&sbuf);
    keylength = sbuf.st_size;
    keybuf = malloc(keylength+8);

    /* Open file and get contents to buffer */
    if(!(key_fd=fopen("/tmp/lastKey.key","r"))) {
	NXRunAlertPanel("Can't copy PGP Key to Pasteboard",
			"Can't open the temporary key file /tmp/lastKey.key",0,0,0);
	return NULL;
    }

    fread(keybuf,keylength,1,key_fd);
    fclose(key_fd);

    /* Put buffer into pasteboard  */
    pb = [Pasteboard new];
    pbTypes[0] = (const char *)NXAsciiPboardType;
    [pb declareTypes:pbTypes num:1 owner:nil];
    [pb writeType:NXAsciiPboardType data:keybuf length:keylength];

    free(keybuf);
    
    return self;
}






/***********************  NOTE   NOTE   NOTE   NOTE   NOTE   ******************/
/*                                                                            */
/*                                                                            */
/*  None of the following methods are implemented.                            */
/*  The parameter types and return types are educated guesses.                */
/*  I encourage anyone and everyone to complete this implementation.          */
/*  Anyone who does contribute, please send your new code to me               */
/*  at this address:                                                          */
/*                                                                            */
/*                    edx@cc.usu.edu                                          */
/*                                                                            */
/*  Thanks!                                                                   */
/*                    Howard Cole                                             */
/*                    USU Research Foundation                                 */
/*                    Space Dynamics Lab                                      */
/*                                                                            */
/*                                                                            */
/******************************************************************************/


- (char *)getPublicKeyFor:(char *)user
{
    fprintf(stderr,"Cryptor: Unimplemented call:to getpublicKeyFor\n");
    return NULL;
}


- (char *)publicKeyFromPrivateKey:(char *)privateKey
{
    fprintf(stderr,"Cryptor: Unimplemented call:to publicKeyFromPrivateKey\n");
    return NULL;
}


- addKeyToKeyring:(char *)key
{
    fprintf(stderr,"Cryptor: Unimplemented call:to addKeyToKeyring:%s\n",key);
    return self;
}





- beginKeyGeneration
{
    return self;
}


- endKeyGeneration
{
    return self;
}


@end

